; Sound.asm: program to play around with the sound HW.
;
;
;

        .include "startup.asm"
	.nomlist
	.list
; ----
; defines
INIT_ADD1	= $8003
PLAY_ADD1	= $8000
INIT_ADD2	= $BF60
PLAY_ADD2	= $8085
INIT_ADD3	= $A003
PLAY_ADD3	= $A000

; Zero-page variables

	.zp
NES_4000: .ds 1
NES_4001: .ds 1
NES_4002: .ds 1
NES_4003: .ds 1
NES_4004: .ds 1
NES_4005: .ds 1
NES_4006: .ds 1
NES_4007: .ds 1
NES_4008: .ds 1
NES_4009: .ds 1
NES_400A: .ds 1
NES_400B: .ds 1
NES_400C: .ds 1
NES_400D: .ds 1
NES_400E: .ds 1
NES_400F: .ds 1
NES_4010: .ds 1
NES_4011: .ds 1
NES_4012: .ds 1
NES_4013: .ds 1
NES_4014: .ds 1
NES_4015: .ds 1
NES_4016: .ds 1
NES_4017: .ds 1

nmi_count: ds	1	;Counter for NMIs...
song_num: .ds	1
ptr:	.ds	2 	; pointer to buffer address
a_cnt:	.ds	1
x_cnt:	.ds	1
cursor:	.ds	1	;Which value is being changed?
duty1: .ds	1
duty2: .ds	1

	;...


;--- CODE area ----------

	.code
	.bank MAIN_BANK
	.org  $C000

main:	
	jsr	setup_video
	map	Mus_Dat3a	;Map the music code into $8000-BFFF
	map	Mus_Dat3b

	stz	<song_num
	stz	<duty1
	stz	<duty2
	jsr	setup_sound
	jsr	Initialize_Music
	jsr	Write_SoundHW
	jsr	print_values

	vsync
	setvec #VSYNC,vsync_proc
	vec_on #VSYNC

.here:	bra    .here

	;...

vsync_proc:
	inc	<nmi_count
	jsr	TestJoypad	;This changes the values and cursor
	jsr	Play_Music
	jsr	Write_SoundHW	;This should change the frequency and volume
	jsr	print_values	;This should print values
	rts

;[ USER DATA ]

Map_NES:
	lda	#$F8
	TAM0
	TAM2
	rts

Map_PCE:
	lda	#$FF
	TAM0
	rts


Initialize_Music:

	jsr	Map_NES		;Change to NES' memory map!
	lda	<song_num
	jsr	INIT_ADD3
	jsr	Map_PCE		;Change back to PCE's memory map!
	rts

Play_Music:
	jsr	Map_NES
	jsr	PLAY_ADD3
	jsr	Map_PCE
	rts


;==============================================================================
TestJoypad:
	tst	#$20,joytrg	;This is RIGHT
	beq	.NoPushRight
	inc	<song_num
	jmp	.Update_Sound
.NoPushRight
	tst	#$80,joytrg	;This is LEFT
	beq	.NoPushLeft
	dec	<song_num
	jmp	.Update_Sound
.NoPushLeft
	rts

.Update_Sound
	jsr	Initialize_Music
	rts
;=======================================================================

Write_SoundHW:
	lda	$4000		;Get duty cycle
	and	#%11000000
	cmp	<duty1		;Get previous duty
	sta	<duty1
	beq	.noduty1change
	lsr	a		;Shift duty down by 1
	tax	;Set up correct waveform.
	cly
	jsr	setup_sound_regs	;0

.noduty1change
	lda	$4004		;Get duty cycle
	and	#%11000000
	cmp	<duty2		;Get previous duty
	sta	<duty2
	beq	.noduty2change
	lsr	a		;Shift duty down by 1
	tax	;Set up correct waveform.
	ldy	#1
	jsr	setup_sound_regs	;1

.noduty2change
;*************************************************

	stz	$0800		;Choose channel 0
	lda	$4003		;Frequency High
	and	#$07
	sta	$0803
	lda	$4002		;Frequency Low
	sta	$0802
	lda	#$F2
	sta	$0805		;Set to left channel

	cmpw	#$0000,<$02	;Is $4002,$4003 set to 0?
	beq	.no_play_sq1	;If so, channel is off!
	lda	$4000		;NES Square 1
	and	#$0F		;Isolate volume
	asl	a
.no_play_sq1:
	ora	#$80
	sta	$0804		;Write PCE Volume
;====================================================
	lda	#$01
	sta	$0800		;Choose channel 1
	lda	$4007		;Frequency High
	and	#$07
	sta	$0803
	lda	$4006		;Frequency Low
	sta	$0802
	lda	#$2F
	sta	$0805		;Set to right channel

	cmpw	#$0000,<$06	;Is $4006,$4007 set to 0?
	beq	.no_play_sq2
	lda	$4004		;NES Square 2
	and	#$0F		;Isolate volume
	asl	a
.no_play_sq2:
	ora	#$80
	sta	$0804		;Write PCE Volume
;====================================================
	lda	#$02
	sta	$0800		;Choose channel 2
	lda	$400A		;Frequency Low
	asl	a
	sta	$0802
	lda	$400B		;Frequency High
	and	#$07
	rol	a
	sta	$0803
	lda	#$AD
	sta	$0805		;Set to left channel

	cmpw	#$0000,<$0A	;Is $400A,$400B set to 0?
	beq	.no_play_tri	;Volume is 0?
;	lda	$4008		;NES Triangle
;	and	#$0F		;Isolate volume
	lda	#$0A
	asl	a
.no_play_tri:
	ora	#$80
	sta	$0804		;Write PCE Volume
;====================================================
	lda	#$04
	sta	$0800		;Choose channel 4	;Noise!
	lda	$400E		;Frequency Low
	and	#$0F		;Isolate noise freq
	tax			;Index it
	lda	noise_table,X
	ora	#$80
	sta	$0807		;NOISE-ONLY REGISTER!

	lda	#$C6
	sta	$0805		;Set to right channel


	lda	$400C		;NES Noise
	and	#$0F		;Isolate volume
	asl	a
	bbr3	<$15,.no_play_noise
	ora	#$80
.no_play_noise
	sta	$0804		;Write PCE Volume
;====================================================
	rts

noise_table:
	db 0,0,0,1,2,3,4,5
	db 6,8,12,16,24,31,31,31

;@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
setup_sound:
	cly
	clx	;Square wave
;	ldx	#192	;Triangle
	jsr	setup_sound_regs	;0
	ldy	#1
	clx	;Square wave
;	ldx	#192	;Triangle
	jsr	setup_sound_regs	;1
	ldy	#2
	ldx	#128	;Triangle
;	ldx	#192	;Triangle
	jsr	setup_sound_regs	;2
	ldy	#3
	clx
	jsr	setup_sound_regs	;3
	ldy	#4
	clx
	jsr	setup_sound_regs	;4
	ldy	#5
	clx
	jsr	setup_sound_regs	;5

	rts


setup_sound_regs:	;Inputs: Y-channel #, X-waveform

	sty	$0800		;Choose channel 0
	lda	#%01000000	;reset the waveform write pointer
	sta	$0804
	lda	#%00000000
	sta	$0804

	cly
load:   lda     wave_tables,x  ; Load a sample in A
	sta     $0806   ; Write the sample
	inx             ; Increment the table index
	iny
	cpy     #32     ; Have we reached the end?
	bne     load    ; If no continue

	stw	#$0000,$0802		;Write a word (frequency)
	lda	#$FF
	sta	$0805				;Equal Balance.
	sta	$0801				;Master Volume
	lda	#$00
	eor	#$00			;We do this by ORing with the volume.
	sta	$0804
	
	

	rts
;-----------------------------------------------
wave_tables:
wave_square0: db 31,31,31,31,0,0,0,0,0,0,0,0,0,0,0,0
	db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
wave_square1: db 30,30,30,30,30,30,30,30,0,0,0,0,0,0,0,0
	db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
wave_square2: db 28,28,28,28,28,28,28,28,28,28,28,28,28,28,28,28
	db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
wave_square3: db 30,30,30,30,30,30,30,30,30,30,30,30,30,30,30,30
	db 30,30,30,30,30,30,30,30,0,0,0,0,0,0,0,0
wave_triangle: db 0,2,4,6,8,10,12,14,16,18,20,22,24,26,28,30
	db 31,29,27,25,23,21,19,17,15,13,11,9,7,5,3,1
wave_sine:   db      18,22,24,26,28,28,30,30
	db      30,30,28,28,26,24,22,18
	db      12,8,6,4,2,2,0,0
	db      0,0,2,2,4,6,8,12
wave_saw:	db	0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15
	db	16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31

;@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

print_values:
	jsr	goto_topleft
	clx
	lda	<song_num
	jsr	print_byte
	addw	#$40,<_di	;jump to next line????
	jsr   	set_write 	; set VRAM write address
	cly
.drawNESRegs
	lda	$4000,Y		;LOAD from RAM IN $4000!!!
	clx
	jsr	print_byte
	tya
	and	#$03		;Check if at next channel
	cmp	#$03
	bne	.no_print_newline
	addw	#$40,<_di	;jump to next line????
	jsr   	set_write 	; set VRAM write address
.no_print_newline
	iny
	cpy	#$18
	bne	.drawNESRegs
	rts

;***************************************************

setup_video:
	map    offchar		; map in the memory bank

	; load blank character into VRAM:

	vload  offchar,#16

	vsync			; vsync to avoid snow
	set_bgpal #0,cgpal,#2	; fill palette #0 and 1


; blank the background

	setvwaddr  $0		; set the VRAM address to $0000

	lda   #32		; size 32 lines tall
.l1:	ldx   #64		; size 64 chars wide
	pha
.l2:	cly

	; Fill each BAT map position with blank character

	stw   #CHAR(offchar),video_data

	dex			; next block
	bne   .l2
	pla
	dec   A 		; next line
	bne   .l1


	; load the characters for 0-9,A-F
	; into $1000 - $1100 in VRAM
	; (each character uses 16 words)

	vload  zero,#16*16

	rts

;*******************************************************************************
goto_topleft:	; print on the background

	lda   #1		; line #1 of 29 (line #0 is not viewable)
	sta   <a_cnt

	cla			; line number in _di
	sta   <_di+1
	lda   <a_cnt
	sta   <_di

	aslw  <_di		; multiply by $40 chars/line, to find address
	aslw  <_di		; of start-of-line (use a 6-bit left-shift)
	aslw  <_di
	aslw  <_di
	aslw  <_di
	aslw  <_di

	jsr   set_write 	; set VRAM write address
	rts

print_byte:	;Prints a byte to the screen.
	pha			; preserve
	lsr   a 		; use top nybble
	lsr   a
	lsr   a
	lsr   a
	sta   video_data	; store char # (0-F) (LSB of VRAM word)
	cpx	#$01
	beq	.noprintred1
	st2   #$01		;Er, print normal bytes as WHITE!
	bra	.nextprintbyte
.noprintred1
	st2   #$11		; MSB of VRAM word: palette #0, base addr $1000
.nextprintbyte
	pla			; restore byte
	and   #$0F		; use bottom nybble
	sta   video_data	; store char # (0-F) (LSB of VRAM word)
	cpx	#$01
	beq	.noprintred2
	st2   #$01		;Er, print normal bytes as WHITE!
	bra	.nextprintbyte2
.noprintred2
	st2   #$11		; MSB of VRAM word: palette #0, base addr $1000
.nextprintbyte2
	rts

;@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

	.bank  MAIN_BANK+1
	.org   $4000

;
; Blank char
;
offchar:  .defchr $0FF0,0,\
	  $00000000,\
	  $00000000,\
	  $00000000,\
	  $00000000,\
	  $00000000,\
	  $00000000,\
	  $00000000,\
	  $00000000

;
; numbers from 0-9,A-F:
;
zero:	  .defchr $1000,0,\
	  $00111110,\
	  $01000011,\
	  $01000101,\
	  $01001001,\
	  $01010001,\
	  $01100001,\
	  $00111110,\
	  $00000000

one:	  .defchr $1010,0,\
	  $00001000,\
	  $00011000,\
	  $00001000,\
	  $00001000,\
	  $00001000,\
	  $00001000,\
	  $00011100,\
	  $00000000

two:	  .defchr $1020,0,\
	  $00111110,\
	  $01000001,\
	  $00000001,\
	  $00111110,\
	  $01000000,\
	  $01000000,\
	  $01111111,\
	  $00000000

three:	  .defchr $1030,0,\
	  $00111110,\
	  $01000001,\
	  $00000001,\
	  $00011110,\
	  $00000001,\
	  $01000001,\
	  $00111110,\
	  $00000000

four:	  .defchr $1040,0,\
	  $00000010,\
	  $00100010,\
	  $00100010,\
	  $00111111,\
	  $00000010,\
	  $00000010,\
	  $00000010,\
	  $00000000

five:	  .defchr $1050,0,\
	  $01111111,\
	  $01000000,\
	  $01000000,\
	  $00111110,\
	  $00000001,\
	  $01000001,\
	  $00111110,\
	  $00000000

six:	  .defchr $1060,0,\
	  $00111110,\
	  $01000000,\
	  $01000000,\
	  $01111110,\
	  $01000001,\
	  $01000001,\
	  $00111110,\
	  $00000000

seven:	  .defchr $1070,0,\
	  $00111111,\
	  $00000001,\
	  $00000001,\
	  $00000010,\
	  $00000100,\
	  $00001000,\
	  $00001000,\
	  $00000000

eight:	  .defchr $1080,0,\
	  $00111110,\
	  $01000001,\
	  $01000001,\
	  $00111110,\
	  $01000001,\
	  $01000001,\
	  $00111110,\
	  $00000000

nine:	  .defchr $1090,0,\
	  $00111110,\
	  $01000001,\
	  $01000001,\
	  $00111111,\
	  $00000001,\
	  $00000001,\
	  $00111110,\
	  $00000000

ten:	  .defchr $10A0,0,\
	  $00111110,\
	  $01000001,\
	  $01000001,\
	  $01111111,\
	  $01000001,\
	  $01000001,\
	  $01000001,\
	  $00000000

eleven:   .defchr $10B0,0,\
	  $01111110,\
	  $01000001,\
	  $01000001,\
	  $01111110,\
	  $01000001,\
	  $01000001,\
	  $01111110,\
	  $00000000

twelve:   .defchr $10C0,0,\
	  $00111110,\
	  $01000001,\
	  $01000000,\
	  $01000000,\
	  $01000000,\
	  $01000001,\
	  $00111110,\
	  $00000000

thirteen: .defchr $10D0,0,\
	  $01111110,\
	  $01000001,\
	  $01000001,\
	  $01000001,\
	  $01000001,\
	  $01000001,\
	  $01111110,\
	  $00000000

fourteen: .defchr $10E0,0,\
	  $01111111,\
	  $01000000,\
	  $01000000,\
	  $01111110,\
	  $01000000,\
	  $01000000,\
	  $01111111,\
	  $00000000

fifteen:  .defchr $10F0,0,\
	  $01111111,\
	  $01000000,\
	  $01000000,\
	  $01111110,\
	  $01000000,\
	  $01000000,\
	  $01000000,\
	  $00000000

;
; Simple palette entry
;
; entry #0 = black, #1-#15 are all white
;
cgpal:	.defpal $000,$777,$777,$777,\
		$777,$777,$777,$777,\
		$777,$777,$777,$777,\
		$777,$777,$777,$777

	.defpal $000,$700,$700,$700,\
		$700,$700,$700,$700,\
		$700,$700,$700,$700,\
		$700,$700,$700,$700

	.bank  MAIN_BANK+2
	.org   $8000
Mus_Dat1a:
;	.incbin "MM2a.bin"

	.bank  MAIN_BANK+3
	.org   $A000
Mus_Dat1b:
;	.incbin "MM2b.bin"



	.bank  MAIN_BANK+4
	.org   $8000
Mus_Dat2a:
;	.incbin "heroa.bin"

	.bank  MAIN_BANK+5
	.org   $A000
Mus_Dat2b:
;	.incbin "herob.bin"


	.bank  MAIN_BANK+6
	.org   $8000
Mus_Dat3a:
	.incbin "zeldaa.bin"

	.bank  MAIN_BANK+7
	.org   $A000
Mus_Dat3b:
	.incbin "zeldab.bin"


	.bank  MAIN_BANK+8
	.org   $8000
Mus_Dat4a:
;	.incbin "nemoa.bin"

	.bank  MAIN_BANK+9
	.org   $A000
Mus_Dat4b:
;	.incbin "nemob.bin"